"""EZSpace provides the namespace for the EZData class. """
#  AGPL-3.0 license
#  Copyright (c) 2025-2026 Asger Jon Vistisen
from __future__ import annotations

from typing import TYPE_CHECKING

from . import EZSpaceHook
from ..mcls import BaseSpace

if TYPE_CHECKING:  # pragma: no cover
  pass


class EZSpace(BaseSpace):
  """
  EZSpace provides the namespace for the EZData class.
  """

  # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
  #  NAMESPACE  # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
  # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

  #  Public Variables
  ezHook = EZSpaceHook()
