"""
WILDCARD sentinel allows overloads to specify that a parameter can be any
value.
"""
#  AGPL-3.0 license
#  Copyright (c) 2025-2026 Asger Jon Vistisen
from __future__ import annotations

from . import Sentinel


class WILDCARD(Sentinel):
  """WILDCARD sentinel allows overloads to specify that a parameter can be
  any value."""
